use super::*;
macro_rules! lex {
    (type ($t:tt), $expected:expr) => {
        let token = Lexer::new(stringify!($t)).scan().expect(stringify!($t:tt should yield a valid token));
        assert_eq!(token.kind, $expected);
    };
    ({ $($t:tt)* }) => {
        Lexer::new(stringify!($($t)*))
    };
}
#[test]
fn ascii_char() {
    lex!(type ('A'), TokenKind::Char('A')); // 'A' should be a valid char
    lex!(type ('\x1b'), TokenKind::Char('\x1b')); // '\\x1b' should be a valid char
}
#[test]
fn unicode_escape_char() {
    lex!(type ('\u{1f988}'), TokenKind::Char('🦈')); // '\\u{1f988}' should be a valid 🦈
}
#[test]
fn number_with_base() {
    lex!(type (0), TokenKind::Number(0, 10)); // 0 should be a 16-bit base-10 number
    lex!(type (42069), TokenKind::Number(42069, 10)); // 42069 should be a 16-bit base-10 number
    lex!(type (0x420), TokenKind::Number(0x420, 16)); // 0x420 should be a 16-bit base-16 number
    lex!(type (0d100), TokenKind::Number(100, 10)); // 0d100 should be a 16-bit base-10 number
    lex!(type (0o100), TokenKind::Number(64, 8)); // 0o100 should be a 16-bit base-8 number
    lex!(type (0b100), TokenKind::Number(4, 2)); // 0b100 should be a 16-bit base-8 number
}
#[test]
fn no_operand_emulated() {
    lex!(type (nop), TokenKind::NoEm(NoEm::Nop)); // nop should be a valid NoEm
    lex!(type (ret), TokenKind::NoEm(NoEm::Ret)); // ret should be a valid NoEm
    lex!(type (clrc), TokenKind::NoEm(NoEm::Clrc)); // clrc should be a valid NoEm
    lex!(type (clrz), TokenKind::NoEm(NoEm::Clrz)); // clrz should be a valid NoEm
    lex!(type (clrn), TokenKind::NoEm(NoEm::Clrn)); // clrn should be a valid NoEm
    lex!(type (setc), TokenKind::NoEm(NoEm::Setc)); // setc should be a valid NoEm
    lex!(type (setz), TokenKind::NoEm(NoEm::Setz)); // setz should be a valid NoEm
    lex!(type (setn), TokenKind::NoEm(NoEm::Setn)); // setn should be a valid NoEm
    lex!(type (dint), TokenKind::NoEm(NoEm::Dint)); // dint should be a valid NoEm
    lex!(type (eint), TokenKind::NoEm(NoEm::Eint)); // eint should be a valid NoEm
}
#[test]
fn registers() {
    lex!(type(pc), TokenKind::Reg(Reg::PC));
    lex!(type(sp), TokenKind::Reg(Reg::SP));
    lex!(type(sr), TokenKind::Reg(Reg::SR));
    lex!(type(cg), TokenKind::Reg(Reg::CG));
    lex!(type(r0), TokenKind::Reg(Reg::PC));
    lex!(type(r1), TokenKind::Reg(Reg::SP));
    lex!(type(r2), TokenKind::Reg(Reg::SR));
    lex!(type(r3), TokenKind::Reg(Reg::CG));
    lex!(type(r4), TokenKind::Reg(Reg::R4));
    lex!(type(r5), TokenKind::Reg(Reg::R5));
    lex!(type(r6), TokenKind::Reg(Reg::R6));
    lex!(type(r7), TokenKind::Reg(Reg::R7));
    lex!(type(r8), TokenKind::Reg(Reg::R8));
    lex!(type(r9), TokenKind::Reg(Reg::R9));
    lex!(type(r10), TokenKind::Reg(Reg::R10));
    lex!(type(r11), TokenKind::Reg(Reg::R11));
    lex!(type(r12), TokenKind::Reg(Reg::R12));
    lex!(type(r13), TokenKind::Reg(Reg::R13));
    lex!(type(r14), TokenKind::Reg(Reg::R14));
    lex!(type(r15), TokenKind::Reg(Reg::R15));
}

// TODO: opcode tests, misc. special character tests, etc.
